use chrono::prelude::*;
use chrono_tz::{Tz, CET};
use std::error::Error;
use tokio::io::{AsyncReadExt, AsyncWriteExt};
use tokio::net::{TcpListener, TcpStream};

const REQ_METHOD: &[u8] = "GET".as_bytes();
const RES_HEADER: &str = "HTTP/1.1 200 OK\nServer: swabd\n";

const PORT: i32 = 9090;

fn to_beat<Tz: TimeZone>(dt: &DateTime<Tz>) -> f32 {
    (dt.hour() * 3600 + dt.minute() * 60 + dt.second()) as f32 / 86.4
}

async fn read_req(sock: &mut TcpStream) -> Result<bool, Box<dyn Error + Send + Sync>> {
    let mut buf = vec![0; 8];

    loop {
        if let Ok(n) = sock.read(&mut buf).await {
            if n >= REQ_METHOD.len() {
                break;
            }
        } else {
            return Err("socket data read error".into());
        }
    }

    Ok(&buf[..REQ_METHOD.len()] == REQ_METHOD)
}

pub async fn serve(port: Option<String>, tz: Option<String>) -> Result<(), Box<dyn Error>> {
    let port = port.and_then(|s| s.parse::<i32>().ok()).unwrap_or(PORT);
    let tz = tz.and_then(|s| s.parse::<Tz>().ok()).unwrap_or(CET);

    let addr = format!("127.0.0.1:{}", port);
    let listen = TcpListener::bind(&addr).await?;

    loop {
        let (mut sock, addr) = listen.accept().await?;

        eprintln!("{}: connected", addr);
        tokio::spawn(async move {
            if let Err(e) = read_req(&mut sock).await {
                eprintln!("{}: {}", addr, e);
            } else {
                let dt = Utc::now().with_timezone(&tz);
                let mut res = format!("@{:.2}\n", to_beat(&dt));

                res = format!("{}Content-Length: {}\n\n{}", RES_HEADER, res.len(), res);
                sock.write_all(res.as_bytes())
                    .await
                    .unwrap_or(eprintln!("{}: connection closed", addr));
            }
        });
    }
}
